class pe_razor(
  $dbpassword = 'razor',
  $pe_tarball_base_url = "https://pm.puppetlabs.com/puppet-enterprise",
  $microkernel_url =
     "https://pm.puppetlabs.com/puppet-enterprise-razor-microkernel-${pe_version}.tar",
  $server_http_port = 8150,
  $server_https_port = 8151
) {
  # Assert the platform being used is one validated for this version of Razor.
  $is_on_el = $::operatingsystem ? {
    'RedHat' => true,
    'CentOS' => true,
    default  => false
  }

  $is_valid_version = $::operatingsystemmajrelease ? {
    6       => true,
    7       => true,
    default => false
  }

  if !($is_on_el and $is_valid_version) {
    crit("Sorry, this version of Razor is only available for Red Hat Enterprise Linux 6 and 7.")
  }
  Pe_java_ks {
    path   => [ '/opt/puppet/bin', '/usr/bin', '/bin', '/usr/sbin', '/sbin' ],
  }

  File {
    ensure => file,
    owner  => 'pe-razor',
    group  => 'pe-razor',
    mode   => '0640'
  }

  # Configuration-like stuff.
  $repo_dir = '/opt/puppet/var/razor/repo'
  $jboss_standalone_dir = '/opt/puppet/share/torquebox/jboss/standalone'
  $jboss_standalone_xml_path = "$jboss_standalone_dir/configuration/standalone.xml"
  # This was taken as an excerpt from puppet_enterprise::params.
  $ssl_dir = '/etc/puppetlabs/puppet/ssl'
  $razor_etc = '/etc/puppetlabs/razor'
  $packages    = '/opt/puppet/razor/packages'

  $java_ssl_alias = 'pe-razor'
  $java_ts_path = "$razor_etc/pe-razor.ts"
  $java_ts_password = "pe-razor"
  $java_ts_alias = "$java_ssl_alias:truststore"
  $java_ks_path = "$razor_etc/pe-razor.ks"
  $java_ks_password = "pe-razor"
  $java_ks_alias = "$java_ssl_alias:keystore"

  # Determine the file extension. Pre-releases are .tar, releases are .tar.gz
  $file_extension = $::pe_build ? {
    /^[^-]+-.+/ => "tar",
    default => "tar.gz",
  }

  # @todo danielp 2013-12-31: This listing will need to be adjusted when more
  # platforms are validated for use with Razor.
  $relarch = "el-${$::operatingsystemmajrelease}-x86_64"
  $build   = "puppet-enterprise-${pe_build}-${relarch}"
  $file    = "${build}.${file_extension}"
  $root    = "${build}/packages/${relarch}"
  $url     = "${pe_tarball_base_url}/${pe_build}/${file}"
  $dest    = "${packages}/razor-repo"

  # Implementation
  file { '/opt/puppet/razor':
    ensure => directory, owner => 'root', group => 'root', mode => '0755'
  }
  file { $packages:
    ensure => directory, owner => 'root', group => 'root', mode => '0755'
  } ->

  # Template uses:
  # - $dest
  # - $file
  # - $url
  # - $root
  exec { "unpack the razor repo":
  # That should allow someone on a 1Mbit link to succeed at this, without
  # hanging *forever* if the network is screwed.  A long time, sure, but
  # not forever.
    provider => shell, timeout => 3600,
    command  => template('pe_razor/install-repo.sh.erb'),
    path     => "/usr/local/bin:/bin:/usr/bin",
    creates  => "${dest}/repodata/repomd.xml.asc"
  } ->

  yumrepo { "pe-razor":
    baseurl  => "file://${dest}",
    enabled  => 1,
    gpgcheck => 0,
  } -> Package <||>

  class { '::pe_razor::database':
    dbpassword => $dbpassword,
  } ->

  package { "pe-razor-server": ensure => latest } ->

  # Configuration
  # This defaults file is fully managed by puppet.
  file { "/opt/puppet/razor/config-defaults.yaml":
    ensure  => file, owner => 'root', mode    => '0444',
    content => template('pe_razor/config-defaults.yaml.erb')
  } ->
  # Only put this here if it does not exist already. The user adds
  # overrides here.
  file { "$razor_etc/config.yaml":
    ensure  => file, owner => 'root',
    content => template('pe_razor/config-defaults.yaml.erb'),
    replace => false
  } ->

  file { "$razor_etc/shiro.ini":
    ensure  => file, owner => 'root',
    content => template('pe_razor/shiro.ini.erb')
  } ->

  file { $repo_dir:
    ensure => directory, mode => '0755'
  } ->

  exec { "migrate the razor database":
    # Allow up to 60 minutes for this to run.  It should be super-fast, right
    # now, but if we get large installations and a migration that needs to do
    # substantial work and can't be lazy-evaluated, this will help.
    provider => shell, timeout => 3600,
    command  => template('pe_razor/do-migrate.sh.erb'),
    unless   => template('pe_razor/do-check-migrate.sh.erb'),
    notify   => Exec["redeploy the razor application to torquebox"]
  } ->

  exec { "unpack the microkernel":
    provider => shell, timeout => 3600,
    command  => template('pe_razor/install-mk.sh.erb'),
    path     => "/usr/local/bin:/bin:/usr/bin",
    creates  => "$repo_dir/microkernel/initrd0.img"
  } ->

  pe_java_ks { $java_ts_alias:
    ensure       => latest,
    target       => $java_ts_path,
    certificate  => "$ssl_dir/certs/ca.pem",
    password     => $java_ts_password,
    trustcacerts => true,
  } ->

  pe_java_ks { $java_ks_alias:
    ensure      => latest,
    target      => $java_ks_path,
    certificate => "$ssl_dir/certs/${::clientcert}.pem",
    private_key => "$ssl_dir/private_keys/${::clientcert}.pem",
    password    => $java_ks_password,
    require     => Pe_java_ks[$java_ts_alias],
  } ->

  file { $jboss_standalone_xml_path:
    ensure => file, owner => 'pe-razor',
    content => template('pe_razor/standalone.xml.erb'),
  } ->

  service { "pe-razor-server": ensure => running, enable => true } ->

  # Ensure that we have deployed the application.
  exec { "deploy the razor application to torquebox":
    provider => shell,
    command  => template('pe_razor/do-deploy.sh.erb'),
    creates  => "$jboss_standalone_dir/deployments/razor-server-knob.yml"
  } ->

  # This one is to work around the fact that `creates` means we never deploy
  # twice.  This one should only trigger if the update to our
  # database happened.
  exec { "redeploy the razor application to torquebox":
    provider    => shell,
    command     => template('pe_razor/do-deploy.sh.erb'),
    refreshonly => true
  }
}
